package day1

import java.nio.file.Files
import java.nio.file.Path
import java.util.stream.Collectors

class Day1 {
    fun runPart1(inputFile: Path): Long {
        return loadFile(inputFile).parallelStream()
            .map {calculateCost(it.toLong())}
            .collect(Collectors.summingLong { it } )
    }

    fun runPart2(inputFile: Path): Long {
        return loadFile(inputFile).parallelStream()
            .map {calculateCostPart2(it.toLong())}
            .collect(Collectors.summingLong { it } )
    }

    private fun calculateCost(x: Long): Long {
        return x.div(3).minus(2)
    }

    private fun calculateCostPart2(x: Long): Long {
        return when (x > 6) {
            false -> 0
            true -> { val temp = calculateCost(x); temp + calculateCostPart2(temp) }
        }
    }

    private fun loadFile(inputFile: Path): List<String> {
        return Files.readAllLines(inputFile)
    }
}
