package day3

import java.nio.file.Files
import java.nio.file.Path
import kotlin.math.abs

class Day3 {
    fun runPart1(inputFile: Path): Int{
        val wires = loadFile(inputFile)
        with(wires[0].intersectsWith(wires[1]).minBy { abs(it.x) + abs(it.y) } ) {
            return abs(this!!.x) + abs(this.y)
        }
    }

    private fun loadFile(inputFile: Path): List<Wire> {
        val origin = Point(0, 0)
        return Files.readAllLines(inputFile).map { wireString ->
            val directions = wireString.split(',')
            var firstPoint = origin
            val returnList = mutableSetOf<WireSegment>()
            directions.forEach { instruction ->
                val secondPoint = firstPoint.withInstruction(Instruction(instruction))
                returnList.add(WireSegment(firstPoint, secondPoint))
                firstPoint = secondPoint
            }
            Wire(returnList)
        }
    }
}

private data class Point(val x: Int, val y: Int) {
    infix fun between(secondPoint: Point): Set<Point> {
        val returnList = mutableSetOf<Point>()
        if (this.x <= secondPoint.x) {
            for (xCoord in this.x..secondPoint.x) {
                if (this.y <= secondPoint.y) {
                    for (yCoord in this.y..secondPoint.y) {
                        returnList.add(Point(xCoord, yCoord))
                    }
                } else {
                    for (yCoord in this.y downTo secondPoint.y) {
                        returnList.add(Point(xCoord, yCoord))
                    }
                }
            }
        } else {
            for (xCoord in this.x downTo secondPoint.x) {
                if (this.y <= secondPoint.y) {
                    for (yCoord in this.y..secondPoint.y) {
                        returnList.add(Point(xCoord, yCoord))
                    }
                } else {
                    for (yCoord in this.y downTo secondPoint.y) {
                        returnList.add(Point(xCoord, yCoord))
                    }
                }
            }
        }
        return returnList
    }
    fun withInstruction(instruction: Instruction): Point {
        return when(instruction.direction) {
            Instruction.Direction.UP -> this.copy(y = this.y + instruction.magnitude)
            Instruction.Direction.DOWN -> this.copy(y = this.y - instruction.magnitude)
            Instruction.Direction.LEFT -> this.copy(x = this.x - instruction.magnitude)
            Instruction.Direction.RIGHT -> this.copy(x = this.x + instruction.magnitude)
        }
    }
}

private class Instruction(instr: String){
    val direction: Direction
    val magnitude: Int
    init {
        direction = instr.first().toDirection()
        magnitude = instr.substring(1).toInt()
    }

    private fun Char.toDirection(): Direction {
        return when (this) {
            'U' -> Direction.UP
            'D' -> Direction.DOWN
            'L' -> Direction.LEFT
            'R' -> Direction.RIGHT
            else -> throw RuntimeException("Impossible")
        }
    }
    enum class Direction {
       UP, RIGHT, LEFT, DOWN
    }
}

private data class WireSegment(val firstPoint: Point, val secondPoint: Point) {
    private val internalPoints: Set<Point> = firstPoint between secondPoint
    infix fun intersectsWith(other: WireSegment): List<Point> {
        return this.internalPoints.filter { it in other.internalPoints }
    }
}

private data class Wire(val wireSegments: MutableCollection<WireSegment>) {
    fun addWire(newWireSegment: WireSegment){
        wireSegments.add(newWireSegment)
    }

    fun intersectsWith(other: Wire): List<Point> {
        val endResult: MutableList<Point> = mutableListOf()
        for (segment in this.wireSegments) {
            for (otherSegment in other.wireSegments) {
                endResult.addAll((segment intersectsWith otherSegment).filter { it != Point(0,0) })
            }
        }
        return endResult
    }
}

